import typer
from wssiham.tools.configuration import Configuration
from wssiham.tools.typerExtend import TyperExtend
from wssiham.tools.utils import Utils
from wssiham.tools.xconfig import Xconfig
from rich.progress import Progress, SpinnerColumn, TextColumn, TimeElapsedColumn
from rich.console import Console
from rich.table import Table
from rich.prompt import IntPrompt

import pathlib
import json
import logging

logger = logging.getLogger(__name__)
app = typer.Typer()

def get_config_file_path(file: str) -> pathlib.Path:
    dirconfile = Xconfig().dirconfile
    return pathlib.Path(f"{dirconfile}/{file}")

def list_config_files() -> list:
    directory = pathlib.Path(Xconfig().dirconfile)
    return sorted(list(directory.glob('*.ini')))

@app.command()
def create(certificat: bool = typer.Option(False, "--cert", "-c", help="Possède un certificat crt"),
           default_wsdl: bool = typer.Option(False, "--wsdl", "-w", help="Générer les chemins wsdl par défaut")):
    """Création d'un fichier de configuration du webservice SIHAM"""
    filename = typer.prompt("Nom de votre fichier de configuration")
    user = typer.prompt("Nom d'utilisateur du webservice")
    password = typer.prompt("Mot de passe du webservice", confirmation_prompt=True, hide_input=True)
    base_wsdl = typer.prompt("Url du webservice (exemple : https://monsiham.univ.fr:8081/)")
    cert = typer.prompt("Chemin du certificat (crt)") if certificat else ""

    Configuration.createws(filename, user, password, base_wsdl, cert, default_wsdl)
    TyperExtend.valid_echo(f'Création du fichier de configuration {filename}.ini')

@app.command()
def listing():
    """Liste les fichiers de configuration existants"""
    files = list_config_files()
    TyperExtend.simple_echo('Liste des fichiers de configurations disponibles dans le répertoire de config')
    TyperExtend.breakline()
    table = Table(title=f'Nombre de configurations trouvées : {len(files)}')
    table.add_column("Chemin du fichier", justify="left", style="cyan", no_wrap=True)
    table.add_column("Fichier", justify="left")
    table.add_column("Dernière modification", justify="left", style="green")
    table.add_column("Configuration par défaut", justify="center", style="red")
    for file in files:
        envname = Xconfig().environnement_name()
        flag = '*' if envname == file.stem else ''
        table.add_row(str(file.parent), file.name, Utils.get_modification_date_file(file), flag)
    console = Console()
    console.print(table)

@app.command()
def cat(file: str = typer.Option("", "--file", "-f", help="fichier de configuration")):
    """Affiche le fichier de configuration par défaut"""
    console = Console()
    if file:
        confile = get_config_file_path(file)
        if not confile.exists() or not confile.suffix == '.ini':
            TyperExtend.error_echo('Le fichier de configuration n\'existe pas ou ne possède pas de .ini')
            raise TyperExtend.exit_unsuccess()
    else:
        confile = Xconfig().pathconf_ws()
        TyperExtend.simple_echo('Affichage de la configuration du fichier par défaut pour l\'accès au webservice SIHAM')
        TyperExtend.breakline()
    console.print_json(json.dumps(Configuration.loadws(str(confile))), indent=4)

@app.command()
def load(file: str = typer.Option("", "--file", "-f", help="nom du fichier de configuration webservice SIHAM à charger")):
    """Charge le fichier de configuration"""
    if file:
        confile = get_config_file_path(file)
        if not confile.exists() or not confile.suffix == '.ini':
            TyperExtend.error_echo('Le fichier de configuration n\'existe pas ou ne possède pas de .ini')
            raise TyperExtend.exit_unsuccess()
    else:
        files = list_config_files()
        TyperExtend.simple_echo('Liste des fichiers de configuration disponibles dans le dossier config')
        TyperExtend.breakline()
        positions = [str(i + 1) for i in range(len(files))]
        for pos, inifile in zip(positions, files):
            TyperExtend.simple_echo(f'{pos}) {inifile.name}')
        TyperExtend.breakline()
        numcomf = IntPrompt.ask("Choisissez la configuration à charger", choices=positions)
        confile = files[int(numcomf) - 1]

    data = Configuration.loadws(str(confile))
    with Progress(SpinnerColumn(), TextColumn("[progress.description]{task.description}"), TimeElapsedColumn(), transient=False) as progress:
        progress.add_task(description="Initialisation des paramètres ...", total=None)
        Configuration.create_confmod(data['ENVIRONMENT_NAME'], f'config/{confile.name}')
    TyperExtend.valid_echo('Chargement réussi !')

@app.command()
def remove(file: str = typer.Option("", "--file", "-f", help="nom du fichier de configuration à supprimer")):
    """Supprime un fichier de configuration"""
    if file:
        confile = get_config_file_path(file)
        if not confile.exists() or not confile.suffix == '.ini':
            TyperExtend.error_echo('Le fichier de configuration n\'existe pas ou ne possède pas de .ini')
            raise TyperExtend.exit_unsuccess()
    else:
        files = list_config_files()
        TyperExtend.simple_echo('Liste des fichiers de configuration disponibles dans le dossier config')
        TyperExtend.breakline()
        positions = [str(i + 1) for i in range(len(files))]
        for pos, inifile in zip(positions, files):
            TyperExtend.simple_echo(f'{pos}) {inifile.name}')
        TyperExtend.breakline()
        numcomf = IntPrompt.ask("Choisissez la configuration à supprimer", choices=positions)
        confile = files[int(numcomf) - 1]

    confile.unlink()
    TyperExtend.valid_echo('Suppression de la configuration réussie !')

@app.callback()
def main() -> None:
    """Conf manager"""
    pass