from dataclasses import dataclass, field
from wssiham.core.recherche_agent.rechercheAgent import RechercheAgent
from datetime import datetime
import logging

logger = logging.getLogger(__name__)


@dataclass
class Doctorant:
    """
    Class représentant un Doctorant au sens Universitaire
    """
    genre: str               # 1 = Monsieur, 2 = Madame
    nom: str
    prenom: str
    date_naissance: str         # format dd/mm/YYYY
    unite_recherche: str

    _numero_etudiant: str = field(init=False, default='')
    _searched: bool = field(init=False, default=False)
    _date_search: datetime = field(init=False, default=None)
    _sihamid: str = field(init=False, default='')
    _crappy_url: str = field(init=False, default='')
    _correspondance: str = field(init=False, default='NON')

    @property
    def sihamid(self):
        return self._sihamid

    @sihamid.setter
    def sihamid(self, value: str) -> None:
        self._sihamid = value

    @property
    def searched(self):
        return self._searched

    @searched.setter
    def searched(self, value: bool) -> None:
        self._searched = value

    @property
    def date_search(self):
        return self._searched

    @date_search.setter
    def date_search(self, value: datetime) -> None:
        self._date_search = value

    @property
    def crappy_url(self):
        return self._crappy_url

    @crappy_url.setter
    def crappy_url(self, value: str) -> None:
        self._crappy_url = value

    @property
    def numero_etudiant(self):
        return self._numero_etudiant

    @numero_etudiant.setter
    def numero_etudiant(self, value: str) -> None:
        self._numero_etudiant = value

    def __del__(self):
        logger.info(f'L\'object {self.__class__.__name__} a été détruit')

    def est_dans_siham(self) -> bool:
        """ Effectue une recherche via le webservice RechercheAgent pour déterminer si le doctorant est dans SIHAM.
        La recherche s'effectue sur le "nom", "prénom" et "date de naissance"

        Si exactement une occurrence est trouvée,

        Returns:
            True: Si au moins une occurence a été trouvée
            False: Si aucune occurence n'a matchée
        """

        def get_first_name(full_name: str) -> str:
            """ Récupérer le premier prénom d'un prénom composé, séparé par un espace ou un trait d'union
            Args:
                full_name (str): Chaine de caractère correspondant à un prénom

            Returns:
                La première chaine de caractère d'une chaine séparé par " " ou "-"
            """
            return full_name.split()[0].split('-')[0]

        def normalizer(data: str) -> str:
            """ Remplace des lettres spécifiques d'une chaine de caractère par un underscore "_"

            Args:
                data (str): Chaine de caractère

            Returns:
                Une chaine de caractère dont des lettres spécifiques ont été remplacé par un undeerscore "_"
            """
            normalmap = {'À': '_', 'Á': '_', 'Â': '_', 'Ã': '_', 'Ä': '_', 'A': '_',
                         'à': '_', 'á': '_', 'â': '_', 'ã': '_', 'ä': '_', 'a': '_',  'ª': '_',
                         'È': '_', 'É': '_', 'Ê': '_', 'Ë': '_', 'E': '_',
                         'è': '_', 'é': '_', 'ê': '_', 'ë': '_', 'e': '_',
                         'Í': '_', 'Ì': '_', 'Î': '_', 'Ï': '_', 'I': '_',
                         'í': '_', 'ì': '_', 'î': '_', 'ï': '_', 'i': '_',
                         'Ò': '_', 'Ó': '_', 'Ô': '_', 'Õ': '_', 'Ö': '_', 'O': '_',
                         'ò': '_', 'ó': '_', 'ô': '_', 'õ': '_', 'ö': '_', 'o': '_', 'º': '_',
                         'Ù': '_', 'Ú': '_', 'Û': '_', 'Ü': '_', 'U': '_',
                         'ù': '_', 'ú': '_', 'û': '_', 'ü': '_', 'u': '_',
                         'Ñ': '_', 'ñ': '_', 'n': '_',
                         'Ç': '_', 'ç': '_', 'C': '_', 'c': '_',
                         '§': '_',  '³': '_', '²': '_', '¹': '_'}
            normalize = str.maketrans(normalmap)

            return data.translate(normalize)

        name = normalizer(self.nom)
        firstname = normalizer(self.prenom)

        result = False
        logger.debug("Début de la recherche de l'individu dans SIHAM")

        # Si le nom est un nom composé
        if "-" in name or " " in name:
            logger.info(f"Nom composé détecté {name}")
            name = get_first_name(name)
            logger.debug(f"Récupération de la première partie du nom {name}")

        # Si le prénom est un prénom composé
        if "-" in firstname or " " in firstname:
            logger.info(f"Prénom composé détecté {firstname}")
            firstname = get_first_name(firstname)
            logger.debug(f"Récupération de la première partie du prénom {firstname}")

        logger.debug(f"Paramètre utilisé : nom={name}, prenom={firstname}, date_naissance={self.date_naissance}")

        search = RechercheAgent(nom_usuel=f"%{name}%",
                                prenom=f'%{firstname}%', date_naissaince=self.date_naissance)

        if search.response():
            self.searched = True
            self.date_search = datetime.now()
            if search.get_nb_agents() == 1:
                self.sihamid = search.response()[0]["donneesPersonnelles"]["matricule"]
                # self.crappy_url = f'https://crappy.univ-reunion.fr/activation/preview?id={self.sihamid}'
            if search.get_nb_agents() > 1:
                logger.warning("Multiple numéro SIHAM trouvé")
            result = True
        return result

    def csvinfo(self) -> str:
        """ Récupère les attributs de PhdStudent pour construire un message csv (séparé par des virgules)

        Returns: Un string contenant les informations de PhdStudent

        """
        return f"{self.nom},{self.prenom},{self.date_naissance},{self.sihamid},{self.crappy_url}\n"

