from dataclasses import dataclass, field
from typing import List, Optional
from wssiham.connection.zeepclient import Zeepclient
from wssiham.model.response.liste_parametrage.genericStructureResponse import StructureEcheanceList
from wssiham.tools.mdate import Mdate

import zeep
import logging


logger = logging.getLogger(__name__)


@dataclass
class RecupStructuresEcheance(Zeepclient):
    """ Ce WebService retourne les codes des structures organisationnelles dans SIHAM dont l’un des attributs échangés
    (Flux - Envoi des structures), gérés par date, entre en vigueur ou arrive à échéance entre les deux dates données
    """

    date_debut_observation: str = field(default='')               # format dd/mm/YYYY
    date_fin_observation: str = field(default='')
    found: tuple[bool, int] = \
        field(init=False, default=(False, -1), repr=False)  # tuple déterminant si un élément a été trouvé et son nombre

    def response(self) -> Optional[StructureEcheanceList]:
        """ Retourne les codes des structures dont l’un des attributs échangés entre en vigueur ou
         arrive à échéance entre les deux dates données


        Returns:
            Liste contenant les informations concernant le (code structure, code Information, type UO, témoin État)

        """
        result = None
        try:
            date_debut_observation = Mdate.string_to_datetime(self.date_debut_observation) if self.date_debut_observation \
            else self.date_debut_observation

            date_fin_observation = Mdate.string_to_datetime(self.date_fin_observation) if self.date_fin_observation \
            else self.date_fin_observation

            resp = \
                self.zeep_client.service.RecupStructuresEcheance(
                    ParamStructures={'dateDebutObservation': date_debut_observation,
                                     'dateFinObservation': date_fin_observation})
            if resp:
                tmp_data = zeep.helpers.serialize_object(resp)
                result = StructureEcheanceList(structure_echeance=tmp_data)
                self.found = (True, len([ele for ele in result.structure_echeance if isinstance(ele, dict)]))
            else:
                self.found = (False, 0)
                logger.debug('Aucun élément trouvé')
        except zeep.exceptions.Fault as fault:
            logger.error(fault)
        return result

