from dataclasses import dataclass, field
from typing import Dict, List, Optional
from wssiham.connection.zeepclient import Zeepclient
from wssiham.model.response.recherche_agent.rechercheAgentResponse import Personnel
import zeep
import logging


logger = logging.getLogger(__name__)


@dataclass
class RechercheAgent(Zeepclient):
    """
    L’objectif de ce flux est de vérifier la présence d’un agent et, s’il existe de renvoyer sa situation.
    """
    nom_usuel: str
    prenom: str

    nom_patronymique: str = field(default='')
    date_naissaince: str = field(default='')                            # format dd/mm/YYYY
    code_etablissement: str = field(default='')                         # code de l'établissement (exemple : 9740478B)
    numero_insee_definitif: str = field(default='', repr=False)         # sans clé NIR

    agents_found: bool = field(init=False, default=False, repr=False)   # la recherche a retourné des agents
    nb_agents_foud: int = field(init=False, default=0, repr=False)      # nombre d'agents trouvé

    def response(self) -> Optional[Personnel]:
        """ Récupère les informations suivantes pour un utilisateur SIHAM recherché :
        - la situation de carrière et contrat, position, affectation, statut d'un agent
        - les données personnelles (état civil, date de naissance) à partir du matricule

        Returns:
            Liste contenant les informations de l'agent recherché sous forme d'OrderedDict.

        """
        result = None
        try:

            resp = self.zeep_client.service.RechercheAgent(
                ParamRechercheAgent=self.param_recherche_agent())
            if resp and not resp[0]['ERREUR_GENERALE']:
                tmp_data = zeep.helpers.serialize_object(resp)
                result = [Personnel(**ele) for ele in tmp_data]
                self.agents_found = True
                self.nb_agents_foud = len(result)
                logger.debug(f"Liste des paramètres utilisés pour ParamRechercheAgent \n"
                             f"{self.param_recherche_agent()}")
        except zeep.exceptions.Fault as fault:
            logger.error(fault)
        return result

    def is_agent_exit(self) -> bool:
        """ Test si l'agent recherché existe ou non

        Returns:
            False : Aucun agent n'a été trouvé
            True : Au moin 1 agent a été trouvé
        """
        return self.agents_found

    def get_nb_agents(self) -> int:
        """ Récupère le nb d'agent trouvé par la recherche

        Returns:
            Le nombre d'agent trouvé par la recherche

        """
        return self.nb_agents_foud

    def param_recherche_agent(self) -> Dict[str, str]:
        """ Paramètre pour l'utilisation du webservice rechercheAgent

        Returns:
            Un dictionnaire contenant les informations (paramètres) nécessaire pour l'utilisation
            du webservice rechercheAgent

        """
        paramra = {'codeEtablissement': self.code_etablissement,
                   'codeNIRSsCle': self.numero_insee_definitif,
                   'dateNaissance': self.date_naissaince,
                   'nomPatronymique': self.nom_patronymique,
                   'nomUsuel': self.nom_usuel,
                   'prenom': self.prenom}

        return paramra

