from dataclasses import dataclass, field
from typing import List, Dict, Union, Any,Optional
from wssiham.connection.zeepclient import Zeepclient

import zeep
import logging


logger = logging.getLogger(__name__)


@dataclass
class RecupUserId(Zeepclient):
    """ L’objectif de ce flux est d’exposer tout ou une partie de la nomenclature RH d’un établissement
    donné aux applications externes.
    """
    matcle: str

    liste_matricule: Any = field(init=False, default=None)

    def __post_init__(self):
        super().__post_init__()
        self.__initialisation_object_liste()

    def response(self) -> List[Optional[Dict[str, Union[str, None, bool]]]]:
        """ Récupère le login d'un matricule SIHAM

        Notes:
            La méthode RecupUserID du webservice utilisateurs de SIHAM permet de récupérer les informations ci-dessous :
                * donneesPersonnelles : Dict, None
                    * dateNaissance: datetime.datetime iso 8601
                    * matricule: str
                    * nomPatronymique: str
                    * nomUsuel: str
                    * numeroInseeDefinitif: str
                    * prenom: str
                    * userID: str, None
                * listeMatriculesErrones : Dict, None
                    * matriculeEnErreur : str
                    * messageErreur : str

        Returns:
            Liste de dictionnaire contenant le matricule SIHAM de l'utilisateur recherché ainsi que son login si disponible
        """
        result = []

        request_payload = {
                'matricules': {
                    'listeMatricules': {
                        'Matricule': {
                            'matcle': self.matcle
                        }
                    }
                }
        }
        try:
            response = self.zeep_client.service.RecupUserID(**request_payload)
            for data in response:
                info = {}
                if data['donneesPersonnelles']:
                    info['matricule'] = data['donneesPersonnelles']['matricule']
                    info['login'] = data['donneesPersonnelles']['userID']
                    info['error'] = False
                else:
                    for error_matricule in data['listeMatriculesErrones']:
                        info['matricule'] = error_matricule['listeMatriculesErrones']['listeMatriculesErrones']['matriculeEnErreur']
                        info['login'] = None
                        info['error'] = True
                result.append(info)
            return result
        except zeep.exceptions.Fault as fault:
            logger.error(fault)
        return result

    def __initialisation_object_liste(self):
        """ Paramètre pour l'utilisation du webservice RecupUserID

        Returns:
            Un dictionnaire contenant les informations (paramètres) nécessaire pour l'utilisation
            du webservice RecupUserID

        """
        t_lmatricule = self.zeep_client.get_type('ns0:Matricule')
        data = {'matcle': self.matcle}
        self.liste_matricule = t_lmatricule(**data)


