from dataclasses import dataclass, field
from datetime import datetime, timedelta
from wssiham.connection.zeepclient import Zeepclient
import zeep
import logging

logger = logging.getLogger(__name__)


@dataclass
class UserSiham(Zeepclient):
    matcle: str  # matricule siham de l'utilisateur

    lognid: str = field(default='')  # login ldap de l'utilisateur
    selfsrv: bool = field(default=False)  # Activation ou non de l'interface d'accès à SIHAM
    adusml: str = field(default='')  # adresse mail de l'utilisateur
    nmprus: str = field(default='')  # Nom usuel

    def create_or_modify(self) -> bool:
        """ Créer ou modifier un utilisateur dans SIHAM et son affectation à un dossier agent existant dans SIHAM.

        Returns
            False : La création de l'accès selfservice HRA n'a pas pu être créée pour le userSIHAM
            True: La création de l'accès selfservice HRA a pu être créée pour le userSIHAM
        """
        result = False
        try:
            response = self.zeep_client.service.createOrModifyUserSiham(self.lognid, self.matcle,
                                                                        self.selfsrv, self.adusml, self.nmprus)
            if response == self.matcle:
                logger.info(f"Création/Modification du matricule {self.matcle} réussie")
                current_day = datetime.today()
                previous_day = current_day - timedelta(days=1)
                if self.selfsrv:
                    logger.debug(f"{self.matcle} : {current_day.strftime('%d-%m-%Y')} -- 31/12/9999")
                else:
                    logger.debug(f"{self.matcle} : {current_day.strftime('%d-%m-%Y')} -- \
                     {previous_day.strftime('%d-%m-%Y')}")
            result = True
        except zeep.exceptions.Fault as fault:
            logger.error(fault)
        return result

    def disable(self) -> bool:
        """ Désactive l'accès à SIHAM à un utilisateur.
        Une date de fin est mis en place dans la partie "authentification" de SIHAM.

        Bug 1 ? : Mettre le selfsrv à **true** ne désactive pas l'accès
        Bug 2 ? : Les habilitations ne sont pas affectés.


        Returns:
            False : La désactivation de l'accès n'a pas pu être faite pour le userSiham
            True: La désactivation de l'accès selfservice HRA a pu être faite pour le userSiham
        """
        result = False
        try:
            response = self.zeep_client.service.disableUserSiham('MATCLE', self.matcle, self.selfsrv)
            if (response is not None) and (response == self.lognid or len(response) == 7):
                logger.info(f"Suppression des accès ({self.matcle}, {self.lognid}) réussie")
                result = True
            if response is None:
                logger.info(f"Aucun retour suite à la suppression \n \
                            Le compte siham-hra a déjà été désactivé ou n'existe pas")
        except zeep.exceptions.Fault as fault:
            logger.error(fault)
            print(fault)
            result = False
        return result
