from __future__ import annotations
import logging
import importlib
from typing import TYPE_CHECKING
from wssiham.tools.utils import Utils

import typer
from typer.core import TyperGroup

if TYPE_CHECKING:
    from click import Context
    from click.core import Command

__version__ = Utils.get_package_version("wssiham")

LAZY_SUBCOMMANDS = {
    "hraaccount": "wssiham.app.hraaccount",
    "indicateur": "wssiham.app.indicateursiham",
    "configuration": "wssiham.app.conf",
    "extraction": "wssiham.app.extraction",
}

logger = logging.getLogger(__name__)

def version_callback(value: bool):
    if value:
        typer.echo(f"Wssiham CLI version: {__version__}")
        raise typer.Exit()

class LazyGroup(TyperGroup):
    """A TyperGroup that allows for lazy loading of subcommands."""

    def __init__(
        self,
        *args,
        lazy_subcommands: dict[str, str] | None = None,
        **kwargs,
    ) -> None:
        """Initialize the LazyGroup."""
        super().__init__(*args, **kwargs)
        self.lazy_subcommands = lazy_subcommands or LAZY_SUBCOMMANDS

    def list_commands(self, ctx: Context) -> list[str]:
        base = super().list_commands(ctx)
        lazy = sorted(self.lazy_subcommands.keys())
        return base + lazy

    def get_command(self, ctx: Context, cmd_name: str) -> Command | None:
        if cmd_name in self.lazy_subcommands:
            cmd = self._lazy_load(cmd_name)
            cmd.info.name = cmd_name
            return typer.main.get_command(cmd)
        return super().get_command(ctx, cmd_name)

    def _lazy_load(self, cmd_name: str) -> typer.Typer:
        module_name = self.lazy_subcommands[cmd_name]
        module = importlib.import_module(module_name)
        app_object = getattr(module, "app", None)  # looks for a Typer instance named 'app'
        if not app_object:
            raise ValueError(
                f"Lazy loading of commands from module {module_name} failed."
            )
        return app_object

app = typer.Typer(no_args_is_help=True, add_completion=False)

@app.callback(cls=LazyGroup)
def main(
    version: bool = typer.Option(None, "--version", callback=version_callback, is_eager=True),
    verbose: bool = False
):
    if verbose:
        logging.getLogger('zeep').setLevel(logging.ERROR)
        logging.getLogger('urllib3').setLevel(logging.ERROR)

        logging.basicConfig(format='%(asctime)s :: %(name)s :: %(levelname)s :: %(message)s',
                            level=logging.DEBUG, handlers=[logging.StreamHandler()])